#!/usr/bin/perl -w

# Extract points achieved by MIR and AMB in each game from zipped results
# files.
# Usage: 
# unzip -c ../competition/results/comp\ 1.zip  | ./getpoints.pl > comp1points
#
# getpoints.pl -b writes the balance (difference between own and oponent's
# points) instead of the strategy's own points.

use strict;

# This function writes the points to a hash and takes care of changing
# non-unique names of opponent strategies.
sub uniqset
{
  my ($hash, $opp, $points)= @_;
  my $num= 2;

  if( !defined($hash->{$opp}) ) {
    $hash->{$opp}= $points;
    return;
  }
  while( defined($hash->{"$opp ($num)"}) ) { ++$num; }
  $hash->{"$opp ($num)"}= $points;
}

my $abs= 1;

if( defined($ARGV[0]) && $ARGV[0] eq "-b" ) {
  shift @ARGV;
  $abs= 0;
}

my $run= 0;
my @mirpts= ( {}, {}, {}, {}, {} );
my @ambpts= ( {}, {}, {}, {}, {} );

while( <> )
{
  if( /^\s*inflating:.*run\s*([0-9]+)/ ) {
    $run= $1 - 1;
    next;
  }
  next unless /^\*\*\* game results/i;
  $_= <>;
  /^\s*player=([^=]+)\s+strategy=([^=(]*)\s+\(.*\)\s+points=(?:([0-9]+)|1,000)\s*$/
  	or die "Unrecognised points line at $.: $_";
  my ($strat1, $points1)= ($2 || $1, $3 || 1000);
  $_= <>;
  /^\s*player=([^=]+)\s+strategy=([^=(]*)\s+\(.*\)\s+points=(?:([0-9]+)|1,000)\s*$/
  	or die "Unrecognised points line at $.: $_";
  my ($strat2, $points2)= ($2 || $1, $3 || 1000);
  if( $strat1 eq "MIR" || $strat2 eq "MIR" ) {
    if( $strat2 ne "MIR" ) {
      uniqset($mirpts[$run], $strat2, $abs? $points1 : $points1-$points2);
    }
    elsif( $strat1 ne "MIR" ) {
      uniqset($mirpts[$run], $strat1, $abs? $points2 : $points2-$points1);
    }
    else {
      uniqset($mirpts[$run], "MIR", $abs? $points1 : $points1-$points2);
      uniqset($mirpts[$run], "self", $abs? $points2 : $points2-$points1);
      # So there will be a MIR line and a self line in the table, with one
      # side's points in the self-match showing up in each.
    }
  }
  if( $strat1 eq "AMB" || $strat2 eq "AMB" ) {
    if( $strat2 ne "AMB" ) {
      uniqset($ambpts[$run], $strat2, $abs? $points1 : $points1-$points2);
    }
    elsif( $strat1 ne "AMB" ) {
      uniqset($ambpts[$run], $strat1, $abs? $points2 : $points2-$points1);
    }
    else {
      uniqset($ambpts[$run], "AMB", $abs? $points1 : $points1-$points2);
      uniqset($ambpts[$run], "self", $abs? $points2 : $points2-$points1);
      # So there will be a AMB line and a self line in the table, with one
      # side's points in the self-match showing up in each.
    }
  }
}

print $abs? "# Points scored by MIR and AMB against given opponent\n" :
      "# Balance of points - score of MIR or AMB minus score of opponent\n",
	"# opponent\t\tMIR\t\tAMB\n";

for $run (0..4) {
  print "# Run ", $run+1, "\n";
  for my $key (keys(%{$mirpts[$run]})) {
    print $key, "\t\t\t", $mirpts[$run]->{$key}, "\t\t",
    			  $ambpts[$run]->{$key}, "\n";
  }
}


