#!/usr/bin/perl -w

# Compute rank and overall score of MIR and AMB and top and bottom scores
# usage: ./rankscores.pl comp1points

use strict;

my $run= -1;
my %results;

# This function writes a value to a hash having array references as values and
# takes care of changing non-unique keys.
sub uniqsetary
{
  my ($hash, $key, $ind, $points)= @_;
  my $num= 2;

  if( !defined($hash->{$key}->[$ind]) ) {
    $hash->{$key}->[$ind]= $points;
    return;
  }
  while( defined($hash->{"$key ($num)"}->[$ind]) ) { ++$num; }
  $hash->{"$key ($num)"}->[$ind]= $points;
}


while( <> )
{
  if( /tournament\s+results/i ) {
    ++$run;
    next;
  }
  next if $run < 0;
  next if /inflating:/;
  next if /^\s*$/;
  next if /^Date:/;
  next if /^Total run time:/;
  next if /^Player\s+Strategy\s+Won\s+Tied\s+Lost\s+Total Points/;
  /^\s*([^\s].*[^\s])\s+\d+\s+\d+\s+\d+\s+(\d+)\.0\s*$/
  	or die "Table line $. in unrecognised format.";
  uniqsetary(\%results, $1, $run, $2);
}

my $npart= scalar(keys(%results));
print "Read results for ", $npart, " participants in ", $run+1,
	" runs.\n\n";
my $ngames= $npart*($run+1);

my @total;

for my $key (keys(%results)) {
  my $sum= 0;
  $#{$results{$key}} == $run
  	or die "Strategy $key only found in ", $#{$results{$key}}+1, " runs";
  map $sum += $_, @{$results{$key}};
  push @total, [ $key, $sum ];
}

@total= sort { $b->[1] <=> $a->[1] } @total;
for my $ind (0..$#total) {
  if( $total[$ind]->[0] =~ /^Volker Schatz/ ) {
    print $total[$ind]->[0], "\nScore ", $total[$ind]->[1], " (",
    	$total[$ind]->[1]/$ngames, " per game), rank ", $ind, " of $npart (",
	$ind/$npart, ")\n\n";
  }
}

print "Top score:\n", $total[0]->[0], "\n", $total[0]->[1], " (",
	$total[0]->[1]/$ngames, " per game)\n\n";
print "Lowest score:\n", $total[$npart-1]->[0], "\n", $total[$npart-1]->[1],
	" (", $total[$npart-1]->[1]/$ngames, " per game)\n\n";

